@@ -54,9 +54,12 @@ class StateSpace(LengthIter):
 
     def __init__(self, factor_sizes: Sequence[int], factor_names: Optional[Sequence[str]] = None):
         super().__init__()
-        # dimension
+        # dimension: [read only]
         self.__factor_sizes = np.array(factor_sizes)
         self.__factor_sizes.flags.writeable = False
+        # multipliers: [read only]
+        self.__factor_multipliers = _dims_multipliers(self.__factor_sizes)
+        self.__factor_multipliers.flags.writeable = False
         # total permutations
         self.__size = int(np.prod(factor_sizes))
         # factor names
@@ -96,6 +99,21 @@ class StateSpace(LengthIter):
         """A list of names of factors handled by this state space"""
         return self.__factor_names
 
+    @property
+    def factor_multipliers(self) -> np.ndarray:
+        """
+        The cumulative product of the factor_sizes used to convert indices to positions, and positions to indices.
+        - The highest values is at the front, the lowest is at the end always being 1.
+        - The size of this vector is: num_factors + 1
+
+        Formulas:
+            * Use broadcasting to get positions:
+                pos = (idx[..., None] % muls[:-1]) // muls[1:]
+            * Use broadcasting to get indices
+                idx = np.sum(pos * muls[1:], axis=-1)
+        """
+        return self.__factor_multipliers
+
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - #
     # Factor Helpers                                                        #
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - #
@@ -137,6 +155,8 @@ class StateSpace(LengthIter):
         Convert a position to an index (or convert a list of positions to a list of indices)
         - positions are lists of integers, with each element < their corresponding factor size
         - indices are integers < size
+
+        TODO: can factor_multipliers be used to speed this up?
         """
         positions = np.moveaxis(positions, source=-1, destination=0)
         return np.ravel_multi_index(positions, self.__factor_sizes)
@@ -146,6 +166,8 @@ class StateSpace(LengthIter):
         Convert an index to a position (or convert a list of indices to a list of positions)
         - indices are integers < size
         - positions are lists of integers, with each element < their corresponding factor size
+
+        TODO: can factor_multipliers be used to speed this up?
         """
         positions = np.array(np.unravel_index(indices, self.__factor_sizes))
         return np.moveaxis(positions, source=0, destination=-1)
@@ -249,14 +271,14 @@ class StateSpace(LengthIter):
         # return everything
         return f_idx, base_factors, num
 
-    def sample_random_factor_traversal(self, f_idx: int = None, base_factors=None, num: int = None, mode='interval') -> np.ndarray:
+    def sample_random_factor_traversal(self, f_idx: int = None, base_factors=None, num: int = None, mode: str = 'interval', start_index: int = 0) -> np.ndarray:
         """
         Sample a single random factor traversal along the
         given factor index, starting from some random base sample.
         """
         f_idx, base_factors, num = self._get_f_idx_and_factors_and_size(f_idx=f_idx, base_factors=base_factors, num=num)
         # generate traversal
-        base_factors[:, f_idx] = get_idx_traversal(self.factor_sizes[f_idx], num_frames=num, mode=mode)
+        base_factors[:, f_idx] = get_idx_traversal(self.factor_sizes[f_idx], num_frames=num, mode=mode, start_index=start_index)
         # return factors (num_frames, num_factors)
         return base_factors
 
@@ -267,7 +289,7 @@ class StateSpace(LengthIter):
 
 
 @lru_cache()
-def _get_step_size(factor_sizes, f_idx):
+def _get_step_size(factor_sizes, f_idx: int):
     # check values
     assert f_idx >= 0
     assert f_idx < len(factor_sizes)
@@ -279,6 +301,20 @@ def _get_step_size(factor_sizes, f_idx):
     return int(np.ravel_multi_index(pos, factor_sizes))
 
 
+def _dims_multipliers(factor_sizes: np.ndarray) -> np.ndarray:
+    factor_sizes = np.array(factor_sizes)
+    assert factor_sizes.ndim == 1
+    return np.append(np.cumprod(factor_sizes[::-1])[::-1], 1)
+
+
+# @try_njit
+# def _idx_to_pos(idxs, dims_mul):
+#     factors = np.expand_dims(np.array(idxs, dtype='int'), axis=-1)
+#     factors = factors % dims_mul[:-1]
+#     factors //= dims_mul[1:]
+#     return factors
+
+
 # ========================================================================= #
 # Hidden State Space                                                        #
 # ========================================================================= #
