@@ -22,13 +22,15 @@
 #  SOFTWARE.
 #  ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~
 
+import logging
 import os
 import os.path
 
-import hydra
 import pytest
 
 import experiment.run as experiment_run
+from experiment.util.hydra_main import hydra_main
+from tests.util import temp_environ
 from tests.util import temp_sys_args
 
 
@@ -37,19 +39,28 @@ from tests.util import temp_sys_args
 # ========================================================================= #
 
 
-@pytest.mark.parametrize('args', [
-    ['run_action=skip'],
-    ['run_action=prepare_data'],
-    ['run_action=train'],
+@pytest.mark.parametrize(('env', 'args'), [
+    # test the standard configs
+    (dict(), ['run_action=skip']),
+    (dict(), ['run_action=prepare_data']),
+    (dict(), ['run_action=train']),
 ])
-def test_experiment_run(args):
+def test_experiment_run(env, args):
+    # show full errors in hydra
     os.environ['HYDRA_FULL_ERROR'] = '1'
 
-    # TODO: why does this not work when config_path is absolute?
-    #      ie. config_path=os.path.join(os.path.dirname(experiment_run.__file__), 'config')
-    with temp_sys_args([experiment_run.__file__, *args]):
-        hydra_main = hydra.main(config_path='config', config_name='config_test')(experiment_run.run_action)
-        hydra_main()
+    # temporarily set the environment and the arguments
+    with temp_environ(env), temp_sys_args([experiment_run.__file__, *args]):
+        # run the hydra experiment
+        # 1. sets the default search path to `experiment/config`
+        # 2. add to the search path with the `DISENT_CONFIGS_PREPEND` and `DISENT_CONFIGS_APPEND` environment variables
+        # 3. enable the ${exit:<msg>} and various other resolvers for omegaconf/hydra
+        hydra_main(
+            callback=experiment_run.run_action,
+            config_name='config_test',
+            log_level=logging.DEBUG,
+        )
+
 
 
 # ========================================================================= #
